package jp.ac.aitech.k16106kk.metaStrategy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.net.GameInfo;

public class winRate {

	// 勝率MAP
	private Map<Agent, Integer> winMap = new HashMap<Agent, Integer>();
	// 済んだゲーム数
	private int gameNum = -1;

	public void initialize(GameInfo gameInfo) {
		for (Agent agent : gameInfo.getAgentList()) {
			if(!winMap.containsKey(agent)) {
				// 自分はMapに含めない
				if(agent != gameInfo.getAgent()) {
					winMap.put(agent, 0);
				}
			}
		}
		if(gameNum < 0) {
			gameNum = 0;
		}
	}

	public void update_winMap(GameInfo gameInfo) {
		// 済んだゲーム数を+1
		gameNum++;

		// 勝利陣営は人間かどうか
		Species winTeam = Species.HUMAN;
		// 勝利陣営を求める
		for (Agent agent : gameInfo.getAliveAgentList()) {
			if(gameInfo.getRoleMap().get(agent) == Role.WEREWOLF) {
				winTeam = Species.WEREWOLF;
				break;
			}
		}

		// 全エージェントの勝利数を更新
		for (Agent agent : gameInfo.getAgentList()) {
			// 自分は無視
			if(agent != gameInfo.getAgent()) {
				/* ロールマップから，自分の役職を取得．その役職のSpecies(陣営)が
				 * 勝利チームと同じなら勝利数をプラス */
				Species agentSpecies = (gameInfo.getRoleMap().get(agent) != Role.POSSESSED) ? gameInfo.getRoleMap().get(agent).getSpecies() : Species.WEREWOLF;
				if(agentSpecies == winTeam) {
					winMap.put(agent, winMap.get(agent)+1);
				}
			}
		}
	}

	// Agent-勝利数 のマップを返す
	public Map<Agent, Integer> getWinMap(){
		return winMap;
	}

	// 最も勝利数の高いAgent(群)を返す
	public List<Agent> getMostWinner(){
		// 勝利数の高いエージェントのリスト(同勝利数のエージェントを返すためにList)
		List<Agent> winner = new ArrayList<Agent>();
		// 調べ終えたうち，最も高い勝利数
		int winN = -1;

		for(Agent agent : winMap.keySet()) {
			// winNよりも，agentの勝利数の方が大きい場合
			if(winN < winMap.get(agent)) {
				// リストを初期化
				winner.clear();
				// winNを更新
				winN = winMap.get(agent);
				// リストに追加
				winner.add(agent);
			}
			// winNとagentの勝利数が同じ場合
			else if (winN == winMap.get(agent)) {
				// リストに追加
				winner.add(agent);
			}
		}
		return winner;
	}

	// 与えられたAgentsの中でもっとも勝利数が高いAgent(群)を返す
	public List<Agent> chooseAgentsGetMostWinner(List<Agent> agents) {
		// 勝利数の高いエージェントのリスト(同勝利数のエージェントを返すためにList)
		List<Agent> winner = new ArrayList<Agent>();
		// 調べ終えたうち，最も高い勝利数
		int winN = -1;

		for(Agent agent : winMap.keySet()) {
			for(Agent a : agents) {
				if(agent == a) {
					// winNよりも，agentの勝利数の方が大きい場合
					if(winN < winMap.get(agent)) {
						// リストを初期化
						winner.clear();
						// winNを更新
						winN = winMap.get(agent);
						// リストに追加
						winner.add(agent);
					}
					// winNとagentの勝利数が同じ場合
					else if (winN == winMap.get(agent)) {
						// リストに追加
						winner.add(agent);
					}
				}
			}
		}

		return winner;
	}

	// ゲーム数を返す
	public int getGameNum() {
		return gameNum;
	}
}



